@@ -9,7 +9,6 @@ import {
   SceneGridRow,
   SceneTimeRange,
   SceneVariableSet,
-  VariableValueSelectors,
   SceneRefreshPicker,
   SceneObject,
   VizPanelMenu,
@@ -17,39 +16,51 @@ import {
   VizPanelState,
   SceneGridItemLike,
   SceneDataLayerProvider,
-  SceneDataLayerControls,
   UserActionEvent,
   SceneObjectState,
+  LocalValueVariable,
 } from '@grafana/scenes';
-import { contextSrv } from 'app/core/core';
+import { isWeekStart } from '@grafana/ui';
+import { K8S_V1_DASHBOARD_API_CONFIG } from 'app/features/dashboard/api/v1';
+import {
+  getDashboardSceneProfilerWithMetadata,
+  enablePanelProfilingForDashboard,
+  getDashboardComponentInteractionCallback,
+} from 'app/features/dashboard/services/DashboardProfiler';
 import { DashboardModel } from 'app/features/dashboard/state/DashboardModel';
 import { PanelModel } from 'app/features/dashboard/state/PanelModel';
-import { DashboardDTO, DashboardDataDTO } from 'app/types';
+import { DashboardDTO, DashboardDataDTO } from 'app/types/dashboard';
 
 import { addPanelsOnLoadBehavior } from '../addToDashboard/addPanelsOnLoadBehavior';
+import { dashboardAnalyticsInitializer } from '../behaviors/DashboardAnalyticsInitializerBehavior';
+import { LoadDashboardOptions } from '../pages/DashboardScenePageStateManager';
 import { AlertStatesDataLayer } from '../scene/AlertStatesDataLayer';
 import { DashboardAnnotationsDataLayer } from '../scene/DashboardAnnotationsDataLayer';
 import { DashboardControls } from '../scene/DashboardControls';
 import { DashboardDataLayerSet } from '../scene/DashboardDataLayerSet';
 import { registerDashboardMacro } from '../scene/DashboardMacro';
+// DashboardPanelProfilingBehavior removed - now using composed SceneRenderProfiler
 import { DashboardReloadBehavior } from '../scene/DashboardReloadBehavior';
 import { DashboardScene } from '../scene/DashboardScene';
-import { DashboardScopesFacade } from '../scene/DashboardScopesFacade';
 import { LibraryPanelBehavior } from '../scene/LibraryPanelBehavior';
 import { VizPanelLinks, VizPanelLinksMenu } from '../scene/PanelLinks';
 import { panelLinksBehavior, panelMenuBehavior } from '../scene/PanelMenuBehavior';
 import { PanelNotices } from '../scene/PanelNotices';
-import { PanelTimeRange } from '../scene/PanelTimeRange';
-import { RowRepeaterBehavior } from '../scene/RowRepeaterBehavior';
-import { AngularDeprecation } from '../scene/angular/AngularDeprecation';
+import { VizPanelHeaderActions } from '../scene/VizPanelHeaderActions';
+import { VizPanelSubHeader } from '../scene/VizPanelSubHeader';
 import { DashboardGridItem, RepeatDirection } from '../scene/layout-default/DashboardGridItem';
 import { DefaultGridLayoutManager } from '../scene/layout-default/DefaultGridLayoutManager';
-import { RowActions } from '../scene/row-actions/RowActions';
+import { RowRepeaterBehavior } from '../scene/layout-default/RowRepeaterBehavior';
+import { RowActions } from '../scene/layout-default/row-actions/RowActions';
+import { RowItem } from '../scene/layout-rows/RowItem';
+import { RowsLayoutManager } from '../scene/layout-rows/RowsLayoutManager';
+import { getIsLazy } from '../scene/layouts-shared/utils';
+import { PanelTimeRange } from '../scene/panel-timerange/PanelTimeRange';
 import { setDashboardPanelContext } from '../scene/setDashboardPanelContext';
+import { DashboardLayoutManager } from '../scene/types/DashboardLayoutManager';
 import { createPanelDataProvider } from '../utils/createPanelDataProvider';
-import { preserveDashboardSceneStateInLocalStorage } from '../utils/dashboardSessionState';
 import { DashboardInteractions } from '../utils/interactions';
-import { getDashboardSceneFor, getVizPanelKeyForPanelId } from '../utils/utils';
+import { getVizPanelKeyForPanelId } from '../utils/utils';
 import { createVariablesForDashboard, createVariablesForSnapshot } from '../utils/variables';
 
 import { getAngularPanelMigrationHandler } from './angularMigration';
@@ -65,18 +76,116 @@ export interface SaveModelToSceneOptions {
   isEmbedded?: boolean;
 }
 
-export function transformSaveModelToScene(rsp: DashboardDTO): DashboardScene {
+type LayoutCreator = (panels: PanelModel[], preload?: boolean) => DashboardLayoutManager;
+
+export interface SceneCreationOptions {
+  /**
+   * When provided, this function is used to create the dashboard body/layout instead of the default v1 behavior.
+   * This allows callers to inject v2 layout strategy.
+   */
+  createLayout?: LayoutCreator;
+  /**
+   * Determines how the dashboard scene is serialized.
+   * @default 'v1'
+   */
+  targetVersion?: 'v1' | 'v2';
+}
+
+// Rows as SceneGridRow within the grid.
+const createDefaultGridLayout: LayoutCreator = (panels, preload) => {
+  return new DefaultGridLayoutManager({
+    grid: new SceneGridLayout({
+      isLazy: getIsLazy(preload),
+      children: createSceneObjectsForPanels(panels),
+    }),
+  });
+};
+
+/**
+ * V2 layout creator - uses RowsLayoutManager when dashboard has rows.
+ * This creates a layout that can be properly serialized to v2 format.
+ */
+export const createV2RowsLayout: LayoutCreator = (panels, preload) => {
+  const hasRows = panels.some((p) => p.type === 'row');
+  if (hasRows) {
+    return createRowsFromPanels(panels);
+  }
+  // Fall back to default grid layout when no rows
+  return createDefaultGridLayout(panels, preload);
+};
+
+export function transformSaveModelToScene(
+  rsp: DashboardDTO,
+  options?: LoadDashboardOptions,
+  sceneOptions?: SceneCreationOptions
+): DashboardScene {
   // Just to have migrations run
   const oldModel = new DashboardModel(rsp.dashboard, rsp.meta);
 
-  const scene = createDashboardSceneFromDashboardModel(oldModel, rsp.dashboard);
+  const scene = createDashboardSceneFromDashboardModel(oldModel, rsp.dashboard, options, sceneOptions);
   // TODO: refactor createDashboardSceneFromDashboardModel to work on Dashboard schema model
-  scene.setInitialSaveModel(rsp.dashboard);
+
+  const apiVersion = config.featureToggles.kubernetesDashboards
+    ? `${K8S_V1_DASHBOARD_API_CONFIG.group}/${K8S_V1_DASHBOARD_API_CONFIG.version}`
+    : undefined;
+
+  scene.setInitialSaveModel(rsp.dashboard, rsp.meta, apiVersion);
 
   return scene;
 }
 
-export function createSceneObjectsForPanels(oldPanels: PanelModel[]): SceneGridItemLike[] {
+function createRowsFromPanels(oldPanels: PanelModel[]): RowsLayoutManager {
+  const rowItems: RowItem[] = [];
+
+  let currentLegacyRow: PanelModel | null = null;
+  let currentRowPanels: DashboardGridItem[] = [];
+
+  for (const panel of oldPanels) {
+    if (panel.type === 'row') {
+      if (!currentLegacyRow && currentRowPanels.length === 0) {
+        // This is the first row, and we have no panels before it. We set currentLegacyRow to the first row.
+        currentLegacyRow = panel;
+      } else if (!currentLegacyRow) {
+        // This is the first row but we have panels before the first row. We should flush the current panels into a row item with header hidden.
+        rowItems.push(
+          new RowItem({
+            title: '',
+            // Hidden header rows must stay expanded; collapsing them would hide the panels entirely.
+            collapse: false,
+            layout: new DefaultGridLayoutManager({
+              grid: new SceneGridLayout({
+                children: currentRowPanels,
+              }),
+            }),
+            hideHeader: true,
+            $behaviors: [],
+          })
+        );
+        currentRowPanels = [];
+
+        currentLegacyRow = panel;
+      } else {
+        // This is a new row. We should flush the current panels into a row item.
+        rowItems.push(createRowItemFromLegacyRow(currentLegacyRow, currentRowPanels));
+        currentRowPanels = [];
+        currentLegacyRow = panel;
+      }
+    } else {
+      currentRowPanels.push(buildGridItemForPanel(panel));
+    }
+  }
+
+  if (currentLegacyRow) {
+    // If there is a row left to process, we should flush it into a row item.
+    rowItems.push(createRowItemFromLegacyRow(currentLegacyRow, currentRowPanels));
+  }
+
+  return new RowsLayoutManager({
+    rows: rowItems,
+  });
+}
+
+function createSceneObjectsForPanels(oldPanels: PanelModel[]): SceneGridItemLike[] {
   // collects all panels and rows
   const panels: SceneGridItemLike[] = [];
 
@@ -171,22 +280,40 @@ function createRowFromPanelModel(row: PanelModel, content: SceneGridItemLike[]):
   });
 }
 
-export function createDashboardSceneFromDashboardModel(oldModel: DashboardModel, dto: DashboardDataDTO) {
+function createRowItemFromLegacyRow(row: PanelModel, panels: DashboardGridItem[]): RowItem {
+  const rowItem = new RowItem({
+    key: getVizPanelKeyForPanelId(row.id),
+    title: row.title,
+    collapse: row.collapsed,
+    layout: new DefaultGridLayoutManager({
+      grid: new SceneGridLayout({
+        // If the row is collapsed it will have panels within the row model.
+        children: (
+          row.panels?.map((p) => buildGridItemForPanel(p instanceof PanelModel ? p : new PanelModel(p))) ?? []
+        ).concat(panels),
+      }),
+    }),
+    repeatByVariable: row.repeat,
+  });
+  return rowItem;
+}
+
+export function createDashboardSceneFromDashboardModel(
+  oldModel: DashboardModel,
+  dto: DashboardDataDTO,
+  options?: LoadDashboardOptions,
+  sceneOptions?: SceneCreationOptions
+) {
   let variables: SceneVariableSet | undefined;
   let annotationLayers: SceneDataLayerProvider[] = [];
   let alertStatesLayer: AlertStatesDataLayer | undefined;
+  const uid = oldModel.uid;
+  const targetVersion = sceneOptions?.targetVersion ?? 'v1';
 
-  if (oldModel.templating?.list?.length) {
-    if (oldModel.meta.isSnapshot) {
-      variables = createVariablesForSnapshot(oldModel);
-    } else {
-      variables = createVariablesForDashboard(oldModel);
-    }
+  if (oldModel.meta.isSnapshot) {
+    variables = createVariablesForSnapshot(oldModel);
   } else {
-    // Create empty variable set
-    variables = new SceneVariableSet({
-      variables: [],
-    });
+    variables = createVariablesForDashboard(oldModel);
   }
 
   if (oldModel.annotations?.list?.length && !oldModel.isSnapshot()) {
@@ -198,6 +325,7 @@ export function createDashboardSceneFromDashboardModel(oldModel: DashboardModel,
         name: a.name,
         isEnabled: Boolean(a.enable),
         isHidden: Boolean(a.hide),
+        placement: a.placement,
       });
     });
   }
@@ -216,67 +344,114 @@ export function createDashboardSceneFromDashboardModel(oldModel: DashboardModel,
     });
   }
 
+  const scopeMeta =
+    config.featureToggles.scopeFilters && oldModel.scopeMeta
+      ? {
+          trait: oldModel.scopeMeta.trait,
+          groups: oldModel.scopeMeta.groups,
+        }
+      : undefined;
+
+  // Create profiler once and reuse to avoid duplicate metadata setting
+  const dashboardProfiler = getDashboardSceneProfilerWithMetadata(oldModel.uid, oldModel.title);
+
+  const enableProfiling =
+    config.dashboardPerformanceMetrics.findIndex((uid) => uid === '*' || uid === oldModel.uid) !== -1;
+  const queryController = new behaviors.SceneQueryController(
+    {
+      enableProfiling,
+    },
+    dashboardProfiler
+  );
+
+  const interactionTracker = new behaviors.SceneInteractionTracker(
+    {
+      enableInteractionTracking: enableProfiling,
+      onInteractionComplete: getDashboardComponentInteractionCallback(oldModel.uid, oldModel.title),
+    },
+    dashboardProfiler
+  );
+
   const behaviorList: SceneObjectState['$behaviors'] = [
     new behaviors.CursorSync({
       sync: oldModel.graphTooltip,
     }),
-    new behaviors.SceneQueryController(),
+    queryController,
+    interactionTracker,
     registerDashboardMacro,
     registerPanelInteractionsReporter,
     new behaviors.LiveNowTimer({ enabled: oldModel.liveNow }),
-    preserveDashboardSceneStateInLocalStorage,
     addPanelsOnLoadBehavior,
-    new DashboardScopesFacade({
-      reloadOnParamsChange: config.featureToggles.reloadDashboardsOnParamsChange && oldModel.meta.reloadOnParamsChange,
-      uid: oldModel.uid,
-    }),
     new DashboardReloadBehavior({
       reloadOnParamsChange: config.featureToggles.reloadDashboardsOnParamsChange && oldModel.meta.reloadOnParamsChange,
-      uid: oldModel.uid,
-      version: oldModel.version,
+      uid,
     }),
   ];
-  const dashboardScene = new DashboardScene({
-    description: oldModel.description,
-    editable: oldModel.editable,
-    preload: dto.preload ?? false,
-    id: oldModel.id,
-    isDirty: false,
-    links: oldModel.links || [],
-    meta: oldModel.meta,
-    tags: oldModel.tags || [],
-    title: oldModel.title,
-    uid: oldModel.uid,
-    version: oldModel.version,
-    body: new DefaultGridLayoutManager({
+
+  if (enableProfiling) {
+    // Analytics aggregator lifecycle management (initialization, observer registration, cleanup)
+    behaviorList.push(dashboardAnalyticsInitializer);
+  }
+  // Will be enabled in the dashboard creation below
+
+  let body: DashboardLayoutManager;
+
+  if (sceneOptions?.createLayout) {
+    // Use injected layout creator (allows callers to specify v2 or custom layout strategy)
+    body = sceneOptions.createLayout(oldModel.panels, dto.preload);
+  } else {
+    // Default v1 layout: DefaultGridLayoutManager
+    body = new DefaultGridLayoutManager({
       grid: new SceneGridLayout({
-        isLazy: !(dto.preload || contextSrv.user.authenticatedBy === 'render'),
+        isLazy: getIsLazy(dto.preload),
         children: createSceneObjectsForPanels(oldModel.panels),
-        $behaviors: [trackIfEmpty],
       }),
-    }),
-    $timeRange: new SceneTimeRange({
-      from: oldModel.time.from,
-      to: oldModel.time.to,
-      fiscalYearStartMonth: oldModel.fiscalYearStartMonth,
-      timeZone: oldModel.timezone,
-      weekStart: oldModel.weekStart,
-      UNSAFE_nowDelay: oldModel.timepicker?.nowDelay,
-    }),
-    $variables: variables,
-    $behaviors: behaviorList,
-    $data: new DashboardDataLayerSet({ annotationLayers, alertStatesLayer }),
-    controls: new DashboardControls({
-      variableControls: [new VariableValueSelectors({}), new SceneDataLayerControls()],
-      timePicker: new SceneTimePicker({}),
-      refreshPicker: new SceneRefreshPicker({
-        refresh: oldModel.refresh,
-        intervals: oldModel.timepicker.refresh_intervals,
-        withText: true,
+    });
+  }
+
+  const dashboardScene = new DashboardScene(
+    {
+      uid,
+      description: oldModel.description,
+      editable: oldModel.editable,
+      preload: dto.preload ?? false,
+      isDirty: false,
+      links: oldModel.links || [],
+      meta: oldModel.meta,
+      tags: oldModel.tags || [],
+      title: oldModel.title,
+      version: oldModel.version,
+      scopeMeta,
+      body,
+      $timeRange: new SceneTimeRange({
+        from: oldModel.time.from,
+        to: oldModel.time.to,
+        fiscalYearStartMonth: oldModel.fiscalYearStartMonth,
+        timeZone: oldModel.timezone,
+        weekStart: isWeekStart(oldModel.weekStart) ? oldModel.weekStart : undefined,
+        UNSAFE_nowDelay: oldModel.timepicker?.nowDelay,
       }),
-      hideTimeControls: oldModel.timepicker.hidden,
-    }),
-  });
+      $variables: variables,
+      $behaviors: behaviorList,
+      $data: new DashboardDataLayerSet({ annotationLayers, alertStatesLayer }),
+      controls: new DashboardControls({
+        timePicker: new SceneTimePicker({
+          quickRanges: oldModel.timepicker.quick_ranges,
+          defaultQuickRanges: config.quickRanges,
+        }),
+        refreshPicker: new SceneRefreshPicker({
+          refresh: oldModel.refresh,
+          intervals: oldModel.timepicker.refresh_intervals,
+          withText: true,
+        }),
+        hideTimeControls: oldModel.timepicker.hidden,
+      }),
+    },
+    targetVersion
+  );
+
+  // Enable panel profiling for this dashboard using the composed SceneRenderProfiler
+  enablePanelProfilingForDashboard(dashboardScene, uid);
 
   return dashboardScene;
 }
@@ -291,9 +466,6 @@ export function buildGridItemForPanel(panel: PanelModel): DashboardGridItem {
 
   const titleItems: SceneObject[] = [];
 
-  if (config.featureToggles.angularDeprecationUI) {
-    titleItems.push(new AngularDeprecation());
-  }
   titleItems.push(
     new VizPanelLinks({
       rawLinks: panel.links,
@@ -309,16 +481,23 @@ export function buildGridItemForPanel(panel: PanelModel): DashboardGridItem {
     key: getVizPanelKeyForPanelId(panel.id),
     title: panel.title?.substring(0, 5000),
     description: panel.description,
-    pluginId: panel.type ?? 'timeseries',
+    pluginId: panel.type ?? 'timeseries', // TODO: should this be `getDefaultPluginId()`?
     options: panel.options ?? {},
     fieldConfig: panel.fieldConfig,
     pluginVersion: panel.pluginVersion,
+    seriesLimit: config.panelSeriesLimit,
     displayMode: panel.transparent ? 'transparent' : undefined,
     // To be replaced with it's own option persited option instead derived
     hoverHeader: !panel.title && !timeOverrideShown,
     hoverHeaderOffset: 0,
     $data: createPanelDataProvider(panel),
     titleItems,
+    headerActions: new VizPanelHeaderActions({
+      hideGroupByAction: !config.featureToggles.panelGroupBy,
+    }),
+    subHeader: new VizPanelSubHeader({
+      hideNonApplicableDrilldowns: !config.featureToggles.perPanelNonApplicableDrilldowns,
+    }),
     $behaviors: [],
     extendPanelContext: setDashboardPanelContext,
     _UNSAFE_customMigrationHandler: getAngularPanelMigrationHandler(panel),
@@ -338,11 +517,27 @@ export function buildGridItemForPanel(panel: PanelModel): DashboardGridItem {
     });
   }
 
-  if (panel.timeFrom || panel.timeShift) {
+  if (panel.timeFrom || panel.timeShift || panel.timeCompare) {
     vizPanelState.$timeRange = new PanelTimeRange({
       timeFrom: panel.timeFrom,
       timeShift: panel.timeShift,
       hideTimeOverride: panel.hideTimeOverride,
+      compareWith: panel.timeCompare,
+    });
+  }
+
+  if (panel.scopedVars && panel.targets?.[0]?.queryType === 'snapshot') {
+    vizPanelState.$variables = new SceneVariableSet({
+      variables: Object.entries(panel.scopedVars).map(
+        ([key, variable]) =>
+          new LocalValueVariable({
+            name: key,
+            value: variable?.value,
+            text: variable?.text,
+            isMulti: true,
+            includeAll: true,
+          })
+      ),
     });
   }
 
@@ -410,17 +605,3 @@ export const convertOldSnapshotToScenesSnapshot = (panel: PanelModel) => {
     panel.snapshotData = [];
   }
 };
-
-function trackIfEmpty(grid: SceneGridLayout) {
-  getDashboardSceneFor(grid).setState({ isEmpty: grid.state.children.length === 0 });
-
-  const sub = grid.subscribeToState((n, p) => {
-    if (n.children.length !== p.children.length || n.children !== p.children) {
-      getDashboardSceneFor(grid).setState({ isEmpty: n.children.length === 0 });
-    }
-  });
-
-  return () => {
-    sub.unsubscribe();
-  };
-}
